@@ -277,7 +277,7 @@ static const unsigned kCommandIndex = 0;
 static const char *kCannotFindListFile = "Cannot find listfile";
 static const char *kIncorrectListFile = "Incorrect item in listfile.\nCheck charset encoding and -scs switch.";
 static const char *kTerminalOutError = "I won't write compressed data to a terminal";
-static const char *kSameTerminalError = "I won't write data and program's messages to same terminal";
+static const char *kSameTerminalError = "I won't write data and program's messages to same stream";
 static const char *kEmptyFilePath = "Empty file path";
 static const char *kCannotFindArchive = "Cannot find archive";
 
@@ -923,7 +923,7 @@ void CArcCmdLineParser::Parse1(const UStringVector &commandStrings,
 
   if (parser[NKey::kAffinity].ThereIs)
   {
-    const UString &s = us2fs(parser[NKey::kAffinity].PostStrings[0]);
+    const UString &s = parser[NKey::kAffinity].PostStrings[0];
     if (!s.IsEmpty())
     {
       UInt32 v = 0;
@@ -1228,8 +1228,26 @@ void CArcCmdLineParser::Parse2(CArcCmdLineOptions &options)
     
     if (isExtractGroupCommand)
     {
-      if (options.StdOutMode && options.IsStdOutTerminal && options.IsStdErrTerminal)
-        throw CArcCmdLineException(kSameTerminalError);
+      if (options.StdOutMode)
+      {
+        if (
+                  options.Number_for_Percents == k_OutStream_stdout
+            // || options.Number_for_Out      == k_OutStream_stdout
+            // || options.Number_for_Errors   == k_OutStream_stdout
+            ||
+            (
+              (options.IsStdOutTerminal && options.IsStdErrTerminal)
+              &&
+              (
+                      options.Number_for_Percents != k_OutStream_disabled
+                // || options.Number_for_Out      != k_OutStream_disabled
+                // || options.Number_for_Errors   != k_OutStream_disabled
+              )
+            )
+           )
+          throw CArcCmdLineException(kSameTerminalError);
+      }
+      
       if (parser[NKey::kOutputDir].ThereIs)
       {
         eo.OutputDir = us2fs(parser[NKey::kOutputDir].PostStrings[0]);
@@ -1302,8 +1320,18 @@ void CArcCmdLineParser::Parse2(CArcCmdLineOptions &options)
 
     if (updateOptions.StdOutMode && updateOptions.EMailMode)
       throw CArcCmdLineException("stdout mode and email mode cannot be combined");
-    if (updateOptions.StdOutMode && options.IsStdOutTerminal)
-      throw CArcCmdLineException(kTerminalOutError);
+    
+    if (updateOptions.StdOutMode)
+    {
+      if (options.IsStdOutTerminal)
+        throw CArcCmdLineException(kTerminalOutError);
+      
+      if (options.Number_for_Percents == k_OutStream_stdout
+          || options.Number_for_Out == k_OutStream_stdout
+          || options.Number_for_Errors == k_OutStream_stdout)
+        throw CArcCmdLineException(kSameTerminalError);
+    }
+    
     if (updateOptions.StdInMode)
       updateOptions.StdInFileName = parser[NKey::kStdIn].PostStrings.Front();
 
